@@ -615,6 +615,12 @@ fatfs_make_root(FATFS_INFO *fatfs, FS_INODE *fs_inode)
     fs_inode->uid = fs_inode->gid = 0;
 	fs_inode->mtime = fs_inode->atime = fs_inode->ctime = fs_inode->dtime = 0;
 
+	if (fs_inode->name == NULL) {
+		fs_inode->name = (FS_NAME *)mymalloc(sizeof(FS_NAME));
+		fs_inode->name->next = NULL;
+	}
+	fs_inode->name->name[0] = '\0';
+
 	for (i = 1; i < fs_inode->direct_count; i++) 
 		fs_inode->direct_addr[i] = 0;
 
@@ -975,6 +981,10 @@ fatfs_inode_lookup(FS_INFO *fs, INUM_T inum)
  * 
  *************************************************************************/
 
+/* 
+ * The UNALLOC flag is not used in FAT because there is no
+ * notion of sparse files
+ */
 static void
 fatfs_file_walk(FS_INFO *fs, FS_INODE *fs_inode, u_int32_t type, u_int16_t id,
     int flags, FS_FILE_WALK_FN action, char *ptr)
@@ -985,6 +995,12 @@ fatfs_file_walk(FS_INFO *fs, FS_INODE *fs_inode, u_int32_t type, u_int16_t id,
 	u_int32_t	len;
 	FS_BUF		*fs_buf;
 
+	/* If the allocated blocks are not wanted, then exit because that is
+	 * all there is with FAT (no sparse) */
+	if ((flags & FS_FLAG_ALLOC) == 0) {
+		return;
+	}
+
 	fs_buf = fs_buf_alloc(fatfs->ssize);
 	if (flags & FS_FLAG_SLACK)
 		size = roundup(fs_inode->size, fatfs->csize * fatfs->ssize);
@@ -996,8 +1012,11 @@ fatfs_file_walk(FS_INFO *fs, FS_INODE *fs_inode, u_int32_t type, u_int16_t id,
 	if ((clust > (fatfs->clustcnt + 1)) &&
 	  (FATFS_ISEOF(clust, fatfs->mask) == 0)) {
 		if (flags & FS_FLAG_NOABORT) {
-			fprintf(stderr, "fatfs_file_walk: Warning: Invalid starting cluster address in Directory Entry (too large): %lu",
-			  (ULONG)clust);
+			if (verbose) {
+				fprintf(logfp, 
+				  "fatfs_file_walk: Warning: Invalid starting cluster address in Directory Entry (too large): %lu",
+			  	  (ULONG)clust);
+			}
 			return;
 		}
 		else {
@@ -1035,8 +1054,11 @@ fatfs_file_walk(FS_INFO *fs, FS_INODE *fs_inode, u_int32_t type, u_int16_t id,
 			if (sbase > fs->last_block) {
 				fs_buf_free(fs_buf);
 				if (flags & FS_FLAG_NOABORT) {
-					fprintf(stderr, "fatfs_file_walk: Invalid sector address in FAT (too large): %lu",
-					  (ULONG)sbase);
+					if (verbose) {
+						fprintf(logfp, 
+						  "fatfs_file_walk: Invalid sector address in FAT (too large): %lu",
+						  (ULONG)sbase);
+					}
 					return;
 				}
 				else
@@ -1151,13 +1173,13 @@ fatfs_fsstat(FS_INFO *fs, FILE *hFile)
 	fprintf(hFile, "\nFAT CONTENTS (in sectors)\n");
 	fprintf(hFile, "--------------------------------------------\n");
 
-	/* 'sstart' marks the begining of the current run to print */
+	/* 'sstart' marks the first sector of the current run to print */
 	sstart = fatfs->firstclustsect;
 
-	/* cycle via cluster */
+	/* cycle via cluster and look at each cluster in the FAT*/
 	for (i = 2; i < fatfs->clustcnt + 1; i++) {
 		
-		/* 'send' marks the end of the current run, which will extend
+		/* 'send' marks the end sector of the current run, which will extend
 		 * when the current cluster continues to the next 
 		 */
 		send = fatfs->firstclustsect + (i - 1) * fatfs->csize - 1;
@@ -1184,7 +1206,8 @@ fatfs_fsstat(FS_INFO *fs, FILE *hFile)
 		}
 
 		/* reset the starting counter */
-		sstart = fatfs->firstclustsect + (i - 1) * fatfs->csize;
+		//sstart = fatfs->firstclustsect + (i - 1) * fatfs->csize;
+		sstart = send + 1;
 	}
 
 	return;
@@ -1295,7 +1318,7 @@ fatfs_open(const char *name, unsigned char ftype)
 {
     char   		*myname = "fatfs_open";
     FATFS_INFO 	*fatfs = (FATFS_INFO *) mymalloc(sizeof(*fatfs));
-    int     	len, datasectcnt;
+    int     	len;
     FS_INFO 	*fs = &(fatfs->fs_info);
 	fatfs_sb	*fatsb;
 
@@ -1360,23 +1383,32 @@ fatfs_open(const char *name, unsigned char ftype)
 		error ("Error: %s is not a FATFS file system (invalid first FAT sector %lu)", 
 		  name, (ULONG)fatfs->firstfatsect);
 
-	/* The data area starts after the FAT */
+	/* The sector of the begining of the data area  - which is 
+	 * after all of the FATs
+	 *
+	 * For FAT12 and FAT16, the data area starts with the root
+	 * directory entries and then the first cluster.  For FAT32,
+	 * the data area starts with clusters and the root directory
+	 * is somewhere in the data area
+	 */
 	fatfs->firstdatasect = fatfs->firstfatsect + 
 		fatfs->sectperfat * fatfs->numfat;
 
-	/* The cluster area is the same as firstdatasect for FAT32, but is
-	 * after the root directory entires for FAT12 and FAT16 
-	 * NOTE: numroot for FAT32 will be 0 */
+	/* The sector where the first cluster is located.  It will be used
+	 * to translate cluster addresses to sector addresses 
+	 *
+	 * For FAT32, the first cluster is the start of the data area and
+	 * it is after the root directory for FAT12 and FAT16.  At this
+	 * point in the program, numroot is set to 0 for FAT32
+	 */
 	fatfs->firstclustsect = fatfs->firstdatasect + 
 	  ((fatfs->numroot * 32 + fatfs->ssize - 1) / fatfs->ssize);
 
 	/* total number of clusters */
-	fatfs->clustcnt = (fatfs->sectors - fatfs->firstdatasect) / 
+	fatfs->clustcnt = (fatfs->sectors - fatfs->firstclustsect) / 
 	  fatfs->csize;
 
-	/* number of sectors in the data area, used for calculations here only */
-	datasectcnt = (fatfs->sectors - fatfs->firstclustsect) / 
-	  fatfs->csize;
+
 
 	/* identify the FAT type by the total number of data clusters
 	 * this calculation is from the MS FAT Overview Doc
@@ -1384,7 +1416,7 @@ fatfs_open(const char *name, unsigned char ftype)
 	if (ftype == MSAUTO_FAT) {
 
 		/* Anything this size is going to be FAT12 */
-		if (datasectcnt < 4085) {
+		if (fatfs->clustcnt < 4085) {
 			ftype = MS12_FAT;
 		}
 		/* by spec, we can determine FAT16 vs FAT32 by the number of
@@ -1402,7 +1434,7 @@ fatfs_open(const char *name, unsigned char ftype)
 
 	/* Some sanity checks */
 	else {
-		if ((ftype == MS12_FAT) && (datasectcnt >= 4085)) 
+		if ((ftype == MS12_FAT) && (fatfs->clustcnt >= 4085)) 
 			error ("Too many sectors for FAT12: try auto-detect mode");
 		
 		if ((ftype == MS32_FAT) && (fatfs->numroot != 0))
