@@ -33,7 +33,7 @@
  *	is \fBufs\fR (Berkeley fast file system). With Linux the default
  *	type is \fBext2fs\fR (second extended file system).
  * .IP \fB-h\fR
- *	Skip over holes in files, so that absolute address information
+ *	Skip over holes (sparse blocks) in files, so that absolute address information
  *	is lost. This option saves space when copying sparse files.
  * .IP "\fB-H\fR (default)"
  *	Copy holes in files as null blocks, so that absolute address
@@ -77,9 +77,9 @@ static void
 usage()
 {
     printf("usage: %s [-hHsvV] [-f fstype] device [inum[-typ[-id]] ...]\n", progname);
-	printf("\t-h: Do not display holes\n");
+	printf("\t-h: Do not display holes in sparse files\n");
 	printf("\t-H: Display holes (default)\n");
-	printf("\t-s: Display slack space as well\n");
+	printf("\t-s: Display slack space at end of file\n");
 	printf("\t-v: verbose to stderr\n");
 	printf("\t-V: Print version\n");
     printf("\t-f fstype: Image file system type\n");
@@ -109,7 +109,7 @@ int     main(int argc, char **argv)
     FS_INFO *fs;
     char   	*cp;
     INUM_T  inum;
-    int     flags = FS_FLAG_ALLOC | FS_FLAG_UNALLOC;
+	int		flags = FS_FLAG_CONT | FS_FLAG_ALLOC | FS_FLAG_UNALLOC;
     int     ch;
     char   	*fstype = DEF_FSTYPE;
 	FS_INODE *inode;
@@ -188,9 +188,7 @@ int     main(int argc, char **argv)
 		if (!inode)
 			error ("error getting inode");
 
-		fs->file_walk(fs, inode, type, id, 
-		  flags | FS_FLAG_CONT | FS_FLAG_ALLOC | FS_FLAG_UNALLOC, 
-			  icat_action, "");
+		fs->file_walk(fs, inode, type, id, flags, icat_action, "");
 
 		fs_inode_free(inode);
     }
